package jalview.ext.forester;

import jalview.datamodel.Sequence;
import jalview.math.MatrixI;

import java.io.IOException;
import java.io.Writer;
import java.text.DecimalFormat;
import java.text.NumberFormat;

import org.forester.evoinference.matrix.distance.DistanceMatrix;
import org.forester.util.ForesterUtil;
import org.forester.util.IllegalFormatUseException;

public class ForesterMatrix implements DistanceMatrix
{
  private final static NumberFormat PHYLIP_FORMATTER = new DecimalFormat(
          "0.000000"); // straight from forester


  private Sequence[] sequences;

  private int width;

  private int height;

  private final double[][] values;

  private String[] identifiers;

  public ForesterMatrix(MatrixI jalviewInputMatrix,
          Sequence[] matrixSequences)
  {
    width = jalviewInputMatrix.width();
    height = jalviewInputMatrix.height();
    values = new double[width][height];

    this.sequences = matrixSequences;


    if (width != height)
    {
      // some kind of warning?
    }



  }


  @Override
  public String getIdentifier(int i)
  {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public int getIndex(String identifier)
  {
    return 0;
  }

  @Override
  public int getSize()
  {
    return values.length;
  }

  @Override
  public double getValue(int col, int row)
  {
    return values[row][col];
  }

  @Override
  public void setIdentifier(int i, String identifier)
  {
    // TODO Auto-generated method stub

  }

  @Override
  public void setValue(int col, int row, double distance)
  {
    values[row][col] = distance;

  }

  @Override
  public StringBuffer toStringBuffer(Format format)
  {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public double[][] getValues()
  {

    return values;
  }

  @Override
  public void write(Writer w) throws IOException // directly copied from
                                                 // forester
  {
    w.write("    ");
    w.write(getSize() + "");
    w.write(ForesterUtil.LINE_SEPARATOR);
    for (int row = 0; row < getSize(); ++row)
    {
      if (!ForesterUtil.isEmpty(getIdentifier(row)))
      {
        w.write(ForesterUtil.pad(getIdentifier(row), 10, ' ', false)
                .toString());
        w.write(' ');
        w.write(' ');
      }
      else
      {
        throw new IllegalFormatUseException(
                "Phylip format does not allow empty identifiers");
      }
      for (int col = 0; col < getSize(); ++col)
      {
        w.write(PHYLIP_FORMATTER.format(getValue(col, row)));
        if (col < (getSize() - 1))
        {
          w.write(' ');
          w.write(' ');
        }
      }
      if (row < (getSize() - 1))
      {
        w.write(ForesterUtil.LINE_SEPARATOR);
      }
    }

  }


}